@@ -3,7 +3,7 @@ from textwrap import dedent
 try:
     from pkgutil import resolve_name
 except ImportError:
-    from pkgutil_resolve_name import resolve_name
+    from pkgutil_resolve_name import resolve_name  # type: ignore
 
 import click
 import twisted.trial.reporter
@@ -13,7 +13,6 @@ from virtue.runner import run
 
 
 class _Reporter(click.ParamType):
-
     name = "reporter"
 
     _BUILT_IN = {
@@ -41,7 +40,7 @@ class _Reporter(click.ParamType):
         return Reporter()
 
 
-@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
+@click.command(context_settings=dict(help_option_names=["-h", "--help"]))  # type: ignore[arg-type]  # noqa: E501
 @click.version_option(prog_name="virtue")
 @click.option(
     "--reporter",
@@ -52,11 +51,13 @@ class _Reporter(click.ParamType):
         Can either be a fully qualified object name (e.g.
         mypackage.MyReporter) or one of the builtin reporter names:
         """,
-    ) + ", ".join(_Reporter._BUILT_IN),
+    )
+    + ", ".join(_Reporter._BUILT_IN),
     type=_Reporter(),
 )
 @click.option(
-    "-x", "--stop-after",
+    "-x",
+    "--stop-after",
     default=None,
     count=True,
     help=(
@@ -68,11 +69,10 @@ class _Reporter(click.ParamType):
 @click.pass_context
 def main(context, **kwargs):
     """
-    virtue discovers and runs tests found in the given objects.
+    `virtue` discovers and runs tests found in the given objects.
 
     Provide it with one or more tests (packages, modules or objects) to run.
 
     """
-
     result = run(**kwargs)
     context.exit(not result.testsRun or not result.wasSuccessful())
