@@ -1,9 +1,20 @@
+"""
+Loaders take a named test case and load the test appropriately.
+"""
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 import itertools
 
-import attr
+from attrs import field, frozen
+
+if TYPE_CHECKING:
+    import twisted.python.modules
+
+    from virtue.locators import ObjectLocator
 
 
-@attr.s
+@frozen
 class AttributeLoader:
     """
     I load a test case by instantiating a class with a given attribute name.
@@ -14,23 +25,29 @@ class AttributeLoader:
     selected test method).
     """
 
-    cls = attr.ib()
-    attribute = attr.ib()
+    cls: type
+    attribute: str
 
     def load(self):
+        """
+        Load as a single test.
+        """
         return [self.cls(self.attribute)]
 
 
-@attr.s
+@frozen
 class ModuleLoader:
     """
     I load a test case by locating tests in the module with the given name.
     """
 
-    locator = attr.ib(repr=False)
-    module = attr.ib()
+    locator: ObjectLocator = field(repr=False)
+    module: twisted.python.modules.PythonModule
 
     def load(self):
+        """
+        Load all test cases in the module.
+        """
         class_loaders = self.locator.locate_in_module(self.module.load())
         return itertools.chain.from_iterable(
             class_loader.load() for class_loader in class_loaders
