@@ -1,21 +1,26 @@
+"""
+Loaders find tests which are referenced by names, preparing them for running.
+"""
 from unittest import TestCase
 import inspect
 
 try:
     from pkgutil import resolve_name
 except ImportError:
-    from pkgutil_resolve_name import resolve_name
+    from pkgutil_resolve_name import resolve_name  # type: ignore
 
+from attrs import define, field
 from twisted.python.modules import getModule as get_module
 from twisted.python.reflect import fullyQualifiedName as fully_qualified_name
 from twisted.trial.runner import filenameToModule as filename_to_module
-import attr
 
 from virtue.loaders import AttributeLoader, ModuleLoader
 
 
 class UnableToLoad(Exception):
-    pass
+    """
+    A test couldn't be loaded.
+    """
 
 
 def prefixed_by(prefix):
@@ -31,15 +36,19 @@ def prefixed_by(prefix):
 
     def prefixed_by_(name, value=None):
         return name.startswith(prefix)
+
     prefixed_by_.__name__ += prefix
     return prefixed_by_
 
 
 def inherits_from_TestCase(attr, cls):
+    """
+    Return true if a class inherits from `unittest.TestCase`.
+    """
     return issubclass(cls, TestCase)
 
 
-@attr.s
+@define
 class ObjectLocator:
     """
     I locate test cases on an object: a package, module or test class.
@@ -68,11 +77,11 @@ class ObjectLocator:
     """
 
     #: Whether an object is a test method or not
-    is_test_method = attr.ib(default=prefixed_by("test"), repr=False)
+    is_test_method = field(default=prefixed_by("test"), repr=False)
     #: Whether an object is a test class or not
-    is_test_class = attr.ib(default=inherits_from_TestCase, repr=False)
+    is_test_class = field(default=inherits_from_TestCase, repr=False)
     #: Whether an object is a test module or not
-    is_test_module = attr.ib(default=prefixed_by("test_"), repr=False)
+    is_test_module = field(default=prefixed_by("test_"), repr=False)
 
     def __attrs_post_init__(self):
         is_cls, self.is_test_class = self.is_test_class, lambda attr, cls: (
@@ -94,7 +103,6 @@ class ObjectLocator:
         modification will be done, so the file must be importable
         without modification.
         """
-
         try:
             obj = resolve_name(name)
         except ValueError:
@@ -132,7 +140,6 @@ class ObjectLocator:
         """
         Attempt to locate the test cases in the given object (of any kind).
         """
-
         # We don't use inspect.getmembers because its predicate only
         # takes the value, not the attr name.
         if inspect.ismodule(obj):
@@ -151,7 +158,6 @@ class ObjectLocator:
         """
         Locate all of the test cases contained in the given package.
         """
-
         for module in get_module(package.__name__).walkModules():
             _, _, name = module.name.rpartition(".")
             if self.is_test_module(name):
@@ -161,7 +167,6 @@ class ObjectLocator:
         """
         Locate all of the test cases contained in the given module.
         """
-
         for attribute, value in inspect.getmembers(module):
             if self.is_test_class(attribute, value):
                 yield from self.locate_in_class(value)
@@ -170,7 +175,6 @@ class ObjectLocator:
         """
         Locate the methods on the given class that are test cases.
         """
-
         for attribute, value in inspect.getmembers(cls):
             if self.is_test_method(attribute, value):
                 yield AttributeLoader(cls=cls, attribute=attribute)
